@@ -1,6 +1,5 @@
 import collections
 import inspect
-import os
 from collections import OrderedDict
 from types import (
     FunctionType,
@@ -78,6 +77,16 @@ class MissingVersionError(Exception):
     """
 
 
+def removesuffix(string: str, suffix: str) -> str:
+    """
+    Can be replaced with str.removesuffix once Py3.8 support is dropped.
+    """
+
+    if string.endswith(suffix):
+        return string[: -len(suffix)]
+    return string
+
+
 class AmberDataSerializer:
     """
     If extending the serializer, change the VERSION property to some unique value
@@ -124,6 +133,10 @@ class AmberDataSerializer:
                     f.write(f"{cls._marker_prefix}{cls.Marker.Name}: {snapshot.name}\n")
                     for data_line in snapshot_data.splitlines(keepends=True):
                         f.write(cls.with_indent(data_line, 1))
+                    if data_line.endswith("\n"):
+                        # splitlines does not split on a terminal/trailing newline, so we must
+                        # account for that manually
+                        f.write(cls.with_indent("", 1))
                     f.write(f"\n{cls._marker_prefix}{cls.Marker.Divider}\n")
 
     @classmethod
@@ -139,7 +152,7 @@ class AmberDataSerializer:
         missing_version = True
 
         try:
-            with open(filepath, "r", encoding=TEXT_ENCODING, newline=None) as f:
+            with open(filepath, encoding=TEXT_ENCODING, newline=None) as f:
                 for line_no, line in enumerate(f):
                     if line.startswith(cls._marker_prefix):
                         marker_key, *marker_rest = line[marker_offset:].split(
@@ -168,7 +181,9 @@ class AmberDataSerializer:
                             if test_name and snapshot_data:
                                 yield Snapshot(
                                     name=test_name,
-                                    data=snapshot_data.rstrip(os.linesep),
+                                    data=removesuffix(snapshot_data, "\r\n")
+                                    if snapshot_data.endswith("\r\n")
+                                    else removesuffix(snapshot_data, "\n"),
                                     tainted=tainted,
                                 )
                             test_name = None
@@ -371,7 +386,7 @@ class AmberDataSerializer:
         return (name for name in dir(data) if not name.startswith("_"))
 
     @classmethod
-    def object_as_named_tuple(cls, data: Any) -> "tuple[Any, ...]":
+    def object_as_named_tuple(cls, data: Any) -> "Tuple[Any, ...]":
         attr_names = list(cls.object_attrs(data))
         return collections.namedtuple(data.__class__.__name__, attr_names)(
             **{prop: getattr(data, prop) for prop in attr_names}
